package jadex.micro.testcases;

import jadex.base.test.TestReport;
import jadex.base.test.Testcase;
import jadex.bridge.modelinfo.Argument;
import jadex.bridge.modelinfo.IArgument;
import jadex.bridge.service.IService;
import jadex.commons.future.IResultListener;
import jadex.micro.MicroAgent;
import jadex.micro.MicroAgentMetaInfo;

import java.util.Collections;

/**
 *  Test searching for services that don't exist. 
 */
public class NoServiceAgent extends MicroAgent
{
	public void executeBody()
	{
		final TestReport	tr	= new TestReport("#1", "Searching for services.");
		
		getServiceContainer().searchServices(INoService.class).addResultListener(new IResultListener()
		{
			public void resultAvailable(Object result)
			{
				if(Collections.EMPTY_LIST.equals(result))
				{
					tr.setSucceeded(true);
				}
				else
				{
					tr.setFailed("Expected empty list but was: "+result);
				}
				setResultValue("testresults", new Testcase(1, new TestReport[]{tr}));
				killAgent();
			}
			
			public void exceptionOccurred(Exception exception)
			{
				tr.setFailed("Exception during test: "+exception);
				setResultValue("testresults", new Testcase(1, new TestReport[]{tr}));
				killAgent();
			}
		});
	}
	
	/** Test service interface. */
	public static interface	INoService	extends IService {}
	
	/**
	 *  Add the 'testresults' marking this agent as a testcase. 
	 */
	public static Object getMetaInfo()
	{
		return new MicroAgentMetaInfo("Test searching for services that don't exist.", 
			null, null, new IArgument[]{new Argument("testresults", null, "Testcase")});
	}
}
